@@ -1,17 +1,18 @@
-import kivy
 from kivy.app import App
 from kivy.uix.widget import Widget
-from kivy.properties import NumericProperty, ReferenceListProperty, ObjectProperty
+from kivy.properties import NumericProperty, ReferenceListProperty,\
+    ObjectProperty
 from kivy.vector import Vector
 from kivy.factory import Factory
 from kivy.clock import Clock
 from random import randint
 
+
 class PongBall(Widget):
     velocity_x = NumericProperty(1)
     velocity_y = NumericProperty(0)
     velocity = ReferenceListProperty(velocity_x, velocity_y)
-    
+
     def move(self):
         self.pos = Vector(*self.velocity) + self.pos
 
@@ -21,7 +22,7 @@ class PongGame(Widget):
 
     def serve_ball(self):
         self.ball.center = self.center
-        self.ball.velocity = Vector(4,0).rotate(randint(0,360))
+        self.ball.velocity = Vector(4, 0).rotate(randint(0, 360))
 
     def update(self, dt):
         self.ball.move()
@@ -39,7 +40,7 @@ class PongApp(App):
     def build(self):
         game = PongGame()
         game.serve_ball()
-        Clock.schedule_interval(game.update, 1.0/60.0)
+        Clock.schedule_interval(game.update, 1.0 / 60.0)
         return game
 
 
