@@ -138,6 +138,16 @@ Available configuration tokens
         property in :class:`~kivy.uix.scrollview.Scrollview` widget.
         Check the widget documentation for more information.
 
+    `scroll_stoptime`: int
+        Default value of :data:`~kivy.uix.scrollview.Scrollview.scroll_stoptime`
+        property in :class:`~kivy.uix.scrollview.Scrollview` widget.
+        Check the widget documentation for more information.
+
+    `scroll_moves`: int
+        Default value of :data:`~kivy.uix.scrollview.Scrollview.scroll_moves`
+        property in :class:`~kivy.uix.scrollview.Scrollview` widget.
+        Check the widget documentation for more information.
+
 :modules:
 
     You can activate modules with this syntax::
@@ -170,11 +180,11 @@ from sys import platform
 from os import environ
 from os.path import exists
 from kivy import kivy_config_fn
-from kivy.logger import Logger
+from kivy.logger import Logger, logger_config_update
 from kivy.utils import OrderedDict
 
 # Version number of current configuration format
-KIVY_CONFIG_VERSION = 6
+KIVY_CONFIG_VERSION = 7
 
 #: Kivy configuration object
 Config = None
@@ -191,6 +201,28 @@ class ConfigParser(PythonConfigParser):
         PythonConfigParser.__init__(self)
         self._sections = OrderedDict()
         self.filename = None
+        self._callbacks = []
+
+    def add_callback(self, callback, section=None, key=None):
+        '''Add a callback to be called when a specific section/key changed. If
+        you don't specify a section or a key, it will call the callback for all
+        section/keys.
+
+        Callbacks will receive 3 arguments: the section, key and value.
+
+        .. versionadded:: 1.4.1
+        '''
+        if section is None and key is not None:
+            raise Exception('You cannot specify a key without a section')
+        self._callbacks.append((callback, section, key))
+
+    def _do_callbacks(self, section, key, value):
+        for callback, csection, ckey in self._callbacks:
+            if csection is not None and csection != section:
+                continue
+            elif ckey is not None and ckey != key:
+                continue
+            callback(section, key, value)
 
     def read(self, filename):
         '''Read only one filename. In contrast to the original ConfigParser of
@@ -206,7 +238,9 @@ class ConfigParser(PythonConfigParser):
         '''Functions similarly to PythonConfigParser's set method, except that
         the value is implicitly converted to a string.
         '''
-        return PythonConfigParser.set(self, section, option, str(value))
+        ret = PythonConfigParser.set(self, section, option, str(value))
+        self._do_callbacks(section, option, str(value))
+        return ret
 
     def setdefaults(self, section, keyvalues):
         '''Set a lot of keys/values in one section at the same time
@@ -264,6 +298,7 @@ if not environ.get('KIVY_DOC_INCLUDE'):
 
     # Create default configuration
     Config = ConfigParser()
+    Config.add_callback(logger_config_update, 'kivy', 'log_level')
 
     # Read config file if exist
     if exists(kivy_config_fn) and \
@@ -363,7 +398,7 @@ if not environ.get('KIVY_DOC_INCLUDE'):
 
         elif version == 3:
             # add token for scrollview
-            Config.setdefault('widgets', 'scroll_timeout', '250')
+            Config.setdefault('widgets', 'scroll_timeout', '55')
             Config.setdefault('widgets', 'scroll_distance', '20')
             Config.setdefault('widgets', 'scroll_friction', '1.')
 
@@ -383,6 +418,13 @@ if not environ.get('KIVY_DOC_INCLUDE'):
         elif version == 5:
             Config.setdefault('graphics', 'resizable', '1')
 
+        elif version == 6:
+            # if the timeout is still the default value, change it
+            if Config.getint('widgets', 'scroll_timeout') == 250:
+                Config.set('widgets', 'scroll_timeout', '55')
+            Config.setdefault('widgets', 'scroll_stoptime', '300')
+            Config.setdefault('widgets', 'scroll_moves', '5')
+
         #elif version == 1:
         #   # add here the command for upgrading from configuration 0 to 1
         #
