@@ -7,7 +7,7 @@ __all__ = ('ImageLoaderPIL', )
 try:
     from PIL import Image as PILImage
 except:
-    raise
+    import Image
 
 from kivy.logger import Logger
 from kivy.core.image import ImageLoaderBase, ImageData, ImageLoader
@@ -54,8 +54,6 @@ class ImageLoaderPIL(ImageLoaderBase):
                 raise
             _img_tmp = imc
 
-        # image are not in the good direction, flip !
-        _img_tmp = _img_tmp.transpose(PILImage.FLIP_TOP_BOTTOM)
         return _img_tmp
 
     def _img_read(self, im):
