@@ -7,10 +7,10 @@ Bubble
 .. image:: images/bubble.jpg
     :align: right
 
-The Bubble widget is a form of menu or a small popup where the options
+The Bubble widget is a form of menu or a small popup where the menu options
 are stacked either vertically or horizontally.
 
-The :class:`Bubble` contains one arrow pointing towards the direction you
+The :class:`Bubble` contains an arrow pointing towards the direction you
 choose.
 
 Simple example
@@ -26,7 +26,7 @@ You can choose the direction the arrow points towards::
 
     Bubble(arrow_pos='top_mid')
 
-The widgets added to Bubble are orderd by default horizintally like in a
+The widgets added to Bubble are orderd by default horizontally as in a
 Boxlayout. You can change that by::
 
     orientation = 'vertical'
@@ -42,7 +42,7 @@ Remove Items::
     or
     bubble.clear_widgets()
 
-Access children list, **Warning** This is important! use content.children to
+Access children list, **Warning** This is important! Use content.children to
 access the children list::
 
     bubble.content.children
@@ -67,9 +67,9 @@ from kivy.properties import ObjectProperty, StringProperty, OptionProperty, \
 
 
 class BubbleButton(Button):
-    '''A button intented to be used as a widget for Bubble widget.
-    You can use "normal" button class, but it will not look good, and you'll
-    need to change yourself the background to be ok.
+    '''A button intended for use as in a Bubble widget.
+    You can use a "normal" button class, but it will not look good, unless
+    the background is changed.
 
     Instead, you can use this BubbleButton widget that already defined the good
     background for you.
@@ -82,7 +82,7 @@ class BubbleContent(GridLayout):
 
 
 class Bubble(GridLayout):
-    '''Bubble class, see module documentation for more information.
+    '''Bubble class. See module documentation for more information.
     '''
 
     background_color = ListProperty([1, 1, 1, 1])
@@ -94,18 +94,18 @@ class Bubble(GridLayout):
 
     border = ListProperty([16, 16, 16, 16])
     '''Border used for :class:`~kivy.graphics.vertex_instructions.BorderImage`
-    graphics instruction, used itself for :data:`background_image`.
+    graphics instruction. Used for :data:`background_image`.
     Can be used when using custom background.
 
-    It must be a list of 4 value: (top, right, bottom, left). Read the
-    BorderImage instruction for more information about how to play with it.
+    It must be a list of 4 values: (top, right, bottom, left). Read the
+    BorderImage instructions for more information about how to use it.
 
     :data:`border` is a :class:`~kivy.properties.ListProperty`, default to (16,
     16, 16, 16)
     '''
 
     background_image = StringProperty('atlas://data/images/defaulttheme/bubble')
-    '''Background image of the bubble
+    '''Background image of the bubble.
 
     :data:`background_image` is a :class:`~kivy.properties.StringProperty`,
     default to 'atlas://data/images/defaulttheme/bubble'.
@@ -113,7 +113,7 @@ class Bubble(GridLayout):
 
     arrow_image = StringProperty(
         'atlas://data/images/defaulttheme/bubble_arrow')
-    ''' Image of the arrow pointing to the bubble
+    ''' Image of the arrow pointing to the bubble.
 
     :data:`arrow_image` is a :class:`~kivy.properties.StringProperty`,
     default to 'atlas://data/images/defaulttheme/bubble_arrow'.
@@ -132,7 +132,7 @@ class Bubble(GridLayout):
     '''
 
     content = ObjectProperty(None)
-    '''This is the object where the main content of the bubble is held
+    '''This is the object where the main content of the bubble is held.
 
     :data:`content` is a :class:`~kivy.properties.ObjectProperty`,
     default to 'None'.
@@ -141,7 +141,7 @@ class Bubble(GridLayout):
     orientation = OptionProperty('horizontal',
             options=('horizontal', 'vertical'))
     '''This specifies the manner in which the children inside bubble
-    are arranged. can be one of 'vertical', 'horizontal'
+    are arranged. Can be one of 'vertical', 'horizontal'
 
     :data:`orientation` is a :class:`~kivy.properties.OptionProperty`,
     default to 'horizontal'.
@@ -157,6 +157,7 @@ class Bubble(GridLayout):
             color=self.background_color)
         self.content = content = BubbleContent(parent=self)
         super(Bubble, self).__init__(**kwargs)
+        content.parent = None
         self.add_widget(content)
         self.on_arrow_pos()
 
@@ -168,7 +169,7 @@ class Bubble(GridLayout):
             or l[0] == self._arrow_layout:
             super(Bubble, self).add_widget(*l)
         else:
-            content.add_widget(l[0])
+            content.add_widget(*l)
 
     def remove_widget(self, *l):
         content = self.content
@@ -222,16 +223,22 @@ class Bubble(GridLayout):
         self_arrow_img.pos = (0, 0)
 
         self.clear_widgets(do_super=True)
+        self_content.parent = None
+
         self_arrow_img.size_hint = (1, None)
         self_arrow_img.height = self_arrow_img.texture_size[1]
         widget_list = []
         arrow_list = []
+        parent = self_arrow_img.parent
+        if parent:
+            parent.remove_widget(self_arrow_img)
+
         if self_arrow_pos[0] == 'b' or self_arrow_pos[0] == 't':
             self.cols = 1
             self.rows = 2
             self_arrow_layout.rows = 1
             self_arrow_layout.cols = 3
-            self_arrow_img.width = self.width/3
+            self_arrow_img.width = self.width / 3
             self_arrow_layout.size_hint = (1, None)
             self_arrow_layout.height = self_arrow_img.height
             if self_arrow_pos[0] == 'b':
@@ -245,12 +252,12 @@ class Bubble(GridLayout):
                         arrow_list = (Widget(), Widget(), self_arrow_img)
                     widget_list = (self_content, self_arrow_layout)
             else:
-                sctr = Scatter(do_translation = False,
-                               rotation = 180,
-                               do_rotation = False,
-                               do_scale = False,
-                               size_hint = (None, None),
-                               size = self_arrow_img.size)
+                sctr = Scatter(do_translation=False,
+                               rotation=180,
+                               do_rotation=False,
+                               do_scale=False,
+                               size_hint=(None, None),
+                               size=self_arrow_img.size)
                 sctr.add_widget(self_arrow_img)
                 if self_arrow_pos == 'top_mid':
                     #add two dummy widgets
@@ -263,31 +270,31 @@ class Bubble(GridLayout):
         elif self_arrow_pos[0] == 'l' or self_arrow_pos[0] == 'r':
             self.cols = 2
             self.rows = 1
-            self_arrow_img.width = self.height/3
+            self_arrow_img.width = self.height / 3
             self_arrow_layout.rows = 3
             self_arrow_layout.cols = 1
             self_arrow_layout.size_hint = (None, 1)
             self_arrow_layout.width = self_arrow_img.height
 
             rotation = -90 if self_arrow_pos[0] == 'l' else 90
-            sctr = Scatter(do_translation = False,
-                               rotation = rotation,
-                               do_rotation = False,
-                               do_scale = False,
-                               size_hint = (None, None),
-                               size = self_arrow_img.size)
+            sctr = Scatter(do_translation=False,
+                               rotation=rotation,
+                               do_rotation=False,
+                               do_scale=False,
+                               size_hint=(None, None),
+                               size=self_arrow_img.size)
             sctr.add_widget(self_arrow_img)
 
             lenarrow_pos = len(self_arrow_pos)
-            if self_arrow_pos[lenarrow_pos-4:] == '_top':
-                arrow_list = (Widget(size_hint = (1, .07)),
-                              sctr, Widget(size_hint = (1, .3)))
-            elif self_arrow_pos[lenarrow_pos-4:] == '_mid':
+            if self_arrow_pos[lenarrow_pos - 4:] == '_top':
+                arrow_list = (Widget(size_hint=(1, .07)),
+                              sctr, Widget(size_hint=(1, .3)))
+            elif self_arrow_pos[lenarrow_pos - 4:] == '_mid':
                 arrow_list = (Widget(), sctr, Widget())
-            elif self_arrow_pos[lenarrow_pos-7:] == '_bottom':
+            elif self_arrow_pos[lenarrow_pos - 7:] == '_bottom':
                 arrow_list = (Widget(), Widget(), sctr)
 
-            if self_arrow_pos[0] =='l':
+            if self_arrow_pos[0] == 'l':
                 widget_list = (self_arrow_layout, self_content)
             else:
                 widget_list = (self_content, self_arrow_layout)
