@@ -23,10 +23,24 @@ Example::
 
     Example1().run()
 
+.. versionchanged:: 1.5.0
+
+    The carousel now support active children, as
+    :class:`~kivy.uix.scrollview.ScrollView`. It will detect a swipe gesture
+    according to :data:`Carousel.scroll_timeout` and
+    :data:`Carousel.scroll_distance`.
+
+    In addition, the container used for adding slide is now hidden in the API.
+    We made a mistake by exposing it to the user. The impacted properties are:
+    :data:`Carousel.slides`, :data:`Carousel.current_slide`,
+    :data:`Carousel.previous_slide`, :data:`Carousel.next_slide`.
+
 '''
 
 __all__ = ('Carousel', )
 
+from functools import partial
+from kivy.clock import Clock
 from kivy.factory import Factory
 from kivy.animation import Animation
 from kivy.uix.stencilview import StencilView
@@ -43,10 +57,15 @@ class Carousel(StencilView):
     '''List of slides inside the carousel. The slides are added when a widget is
     added to Carousel using add_widget().
 
-    :data:`slides` is a list of :class:`~kivy.ui.relativelayout.RelativeLayout`
-    widgets containing the content added through add_widget.
+    :data:`slides` is a :class:`~kivy.properties.ListProperty`, read-only.
     '''
 
+    def _get_slides_container(self):
+        return [x.parent.parent for x in self.slides]
+
+    slides_container = AliasProperty(_get_slides_container, None,
+            bind=('slides', ))
+
     direction = OptionProperty('right',
             options=('right', 'left', 'top', 'bottom'))
     '''Specifies the direction in which the slides are orderd / from
@@ -132,6 +151,11 @@ class Carousel(StencilView):
     the previous slide towards the bottom.
 
     :data:`previous_slide` is a :class:`~kivy.properties.AliasProperty`.
+
+    .. versionchanged:: 1.5.0
+
+        The property doesn't expose the container used for storing the slide.
+        It will now return the real widget you added.
     '''
 
     def _curr_slide(self):
@@ -139,10 +163,13 @@ class Carousel(StencilView):
             return self.slides[self.index]
     current_slide = AliasProperty(_curr_slide, None, bind=('slides', 'index'))
     '''The currently shown slide.
-    Slidesare a :class:`~kivy.uix.relativelayout.Relativelayout` which
-    contains the widget added to the carousel.
 
-    :data:`current_slide` is a :class:`~kivy.properties.AliasProperty`.
+    :data:`current_slide` is an :class:`~kivy.properties.AliasProperty`.
+
+    .. versionchanged:: 1.5.0
+
+        The property doesn't expose the container used for storing the slide.
+        It will now return the real widget you added.
     '''
 
     def _next_slide(self):
@@ -164,6 +191,35 @@ class Carousel(StencilView):
     the previous slide is towards the top.
 
     :data:`previous_slide` is a :class:`~kivy.properties.AliasProperty`.
+
+    .. versionchanged:: 1.5.0
+
+        The property doesn't expose the container used for storing the slide.
+        It will now return the real widget you added.
+    '''
+
+    scroll_timeout = NumericProperty(200)
+    '''Timeout allowed to trigger the :data:`scroll_distance`, in milliseconds.
+    If the user has not moved :data:`scroll_distance` within the timeout,
+    the scrolling will be disabled, and the touch event will go to the children.
+
+    :data:`scroll_timeout` is a :class:`~kivy.properties.NumericProperty`,
+    default to 200 (milliseconds)
+
+    .. versionadded:: 1.5.0
+    '''
+
+    scroll_distance = NumericProperty('20dp')
+    '''Distance to move before scrolling the :class:`ScrollView`, in pixels. As
+    soon as the distance has been traveled, the :class:`ScrollView` will start
+    to scroll, and no touch event will go to children.
+    It is advisable that you base this value on the dpi of your target device's
+    screen.
+
+    :data:`scroll_distance` is a :class:`~kivy.properties.NumericProperty`,
+    default to 20dp.
+
+    .. versionadded:: 1.5.0
     '''
 
     #### private properties, for internal use only ###
@@ -172,14 +228,27 @@ class Carousel(StencilView):
     _current = ObjectProperty(None, allownone=True)
     _next = ObjectProperty(None, allownone=True)
     _offset = NumericProperty(0)
-    _drag_touches = ListProperty([])
+    _touch = ObjectProperty(None, allownone=True)
+
+    def get_slide_container(self, slide):
+        return slide.parent.parent
 
     def _insert_visible_slides(self):
-        self._prev = self.previous_slide
-        self._current = self.current_slide
-        self._next = self.next_slide
-        for slide in self.slides:
-            super(Carousel, self).remove_widget(slide)
+        get_slide_container = self.get_slide_container
+        if self.previous_slide:
+            self._prev = get_slide_container(self.previous_slide)
+        else:
+            self._prev = None
+        if self.current_slide:
+            self._current = get_slide_container(self.current_slide)
+        else:
+            self._current = None
+        if self.next_slide:
+            self._next = get_slide_container(self.next_slide)
+        else:
+            self._next = None
+        for container in self.slides_container:
+            super(Carousel, self).remove_widget(container)
         if self._prev:
             super(Carousel, self).add_widget(self._prev)
         if self._next:
@@ -210,7 +279,7 @@ class Carousel(StencilView):
                 self._next.pos = (self.x, y_next[self.direction])
 
     def on_size(self, *args):
-        for slide in self.slides:
+        for slide in self.slides_container:
             slide.size = self.size
         self._position_visible_slides()
 
@@ -220,7 +289,8 @@ class Carousel(StencilView):
         self._offset = 0
 
     def on_slides(self, *args):
-        self.index = self.index % len(self.slides)
+        if self.slides:
+            self.index = self.index % len(self.slides)
         self._insert_visible_slides()
         self._position_visible_slides()
 
@@ -248,10 +318,8 @@ class Carousel(StencilView):
             if self._offset >= self.height:
                 self.index = self.index + 1
 
-    def on__drag_touches(self, *args):
+    def _start_animation(self, *args):
         Animation.cancel_all(self)
-        if self._drag_touches:
-            return
 
         # compute target offset for ease back, next or prev
         new_offset = 0
@@ -282,52 +350,124 @@ class Carousel(StencilView):
         anim = Animation(_offset=new_offset, d=dur, t='out_quad')
         anim.start(self)
 
+    def _get_uid(self, prefix='sv'):
+        return '{0}.{1}'.format(prefix, self.uid)
+
     def on_touch_down(self, touch):
         if not self.collide_point(*touch.pos):
+            touch.ud[self._get_uid('cavoid')] = True
             return
-        if not super(Carousel, self).on_touch_down(touch):
-            self._drag_touches.append(touch.uid)
-            touch.grab(self)
+        if self._touch:
+            return super(Carousel, self).on_touch_down(touch)
+        Animation.cancel_all(self)
+        self._touch = touch
+        uid = self._get_uid()
+        touch.grab(self)
+        touch.ud[uid] = {
+            'mode': 'unknown',
+            'time': touch.time_start}
+        Clock.schedule_once(self._change_touch_mode,
+                self.scroll_timeout / 1000.)
         return True
 
     def on_touch_move(self, touch):
-        if touch.grab_current is self:
-            if self._drag_touches.index(touch.uid) > 0:
-                return True
-            if self.direction in ['right', 'left']:
-                self._offset += touch.dx
-            if self.direction in ['top', 'bottom']:
-                self._offset += touch.dy
+        if self._get_uid('cavoid') in touch.ud:
+            return
+        if self._touch is not touch:
+            super(Carousel, self).on_touch_move(touch)
+            return self._get_uid() in touch.ud
+        if touch.grab_current is not self:
             return True
-
-        if self.collide_point(*touch.pos):
-            return super(Carousel, self).on_touch_move(touch)
+        ud = touch.ud[self._get_uid()]
+        direction = self.direction
+        if ud['mode'] == 'unknown':
+            if direction in ('right', 'left'):
+                distance = abs(touch.ox - touch.x)
+            else:
+                distance = abs(touch.oy - touch.y)
+            if distance > self.scroll_distance:
+                Clock.unschedule(self._change_touch_mode)
+                ud['mode'] = 'scroll'
+        else:
+            if direction in ('right', 'left'):
+                self._offset += touch.dx
+            if direction in ('top', 'bottom'):
+                self._offset += touch.dy
+        return True
 
     def on_touch_up(self, touch):
-        if touch.grab_current is self:
-            self._drag_touches.remove(touch.uid)
+        if self._get_uid('cavoid') in touch.ud:
+            return
+        if self in [x() for x in touch.grab_list]:
             touch.ungrab(self)
-            return True
+            self._touch = None
+            ud = touch.ud[self._get_uid()]
+            if ud['mode'] == 'unknown':
+                Clock.unschedule(self._change_touch_mode)
+                super(Carousel, self).on_touch_down(touch)
+                Clock.schedule_once(partial(self._do_touch_up, touch), .1)
+            else:
+                self._start_animation()
 
-        if self.collide_point(*touch.pos):
-            return super(Carousel, self).on_touch_move(touch)
+        else:
+            if self._touch is not touch and self.uid not in touch.ud:
+                super(Carousel, self).on_touch_up(touch)
+        return self._get_uid() in touch.ud
 
-    def add_widget(self, *args, **kwargs):
+    def _do_touch_up(self, touch, *largs):
+        super(Carousel, self).on_touch_up(touch)
+        # don't forget about grab event!
+        for x in touch.grab_list[:]:
+            touch.grab_list.remove(x)
+            x = x()
+            if not x:
+                continue
+            touch.grab_current = x
+            super(Carousel, self).on_touch_up(touch)
+        touch.grab_current = None
+
+    def _change_touch_mode(self, *largs):
+        if not self._touch:
+            return
+        self._start_animation()
+        uid = self._get_uid()
+        touch = self._touch
+        ud = touch.ud[uid]
+        if ud['mode'] == 'unknown':
+            touch.ungrab(self)
+            self._touch = None
+            touch.push()
+            touch.apply_transform_2d(self.to_widget)
+            touch.apply_transform_2d(self.to_parent)
+            super(Carousel, self).on_touch_down(touch)
+            touch.pop()
+            return
+
+    def add_widget(self, widget, index=0):
         slide = RelativeLayout(size=self.size, x=self.x - self.width, y=self.y)
-        slide.add_widget(*args, **kwargs)
-        super(Carousel, self).add_widget(slide)
-        self.slides.append(slide)
+        slide.add_widget(widget)
+        super(Carousel, self).add_widget(slide, index)
+        if index != 0:
+            self.slides.insert(index, widget)
+        else:
+            self.slides.append(widget)
 
     def remove_widget(self, widget, *args, **kwargs):
         # XXX be careful, the widget.parent.parent refer to the RelativeLayout
         # added in add_widget(). But it will break if RelativeLayout
         # implementation change.
-        if widget.parent.parent in self.slides:
+        # if we passed the real widget
+        if widget in self.slides:
             slide = widget.parent.parent
-            self.slides.remove(slide)
+            self.slides.remove(widget)
             return slide.remove_widget(widget, *args, **kwargs)
         return super(Carousel, self).remove_widget(widget, *args, **kwargs)
 
+    def clear_widgets(self):
+        for slide in self.slides[:]:
+            self.remove_widget(slide)
+        super(Carousel, self).clear_widgets()
+
 
 if __name__ == '__main__':
     from kivy.app import App
